/*
 * Copyright (c) 2004, 2005, ADAPT project partners (IST-2001-37126).
 * 
 * CheckStockServlet.java
 */

package org.eu.adapt.demonstrator.web.demo;

import java.io.*;
import java.net.URL;
import javax.servlet.*;
import javax.servlet.http.*;
import org.eu.adapt.demonstrator.util.UniqueIdentifierGenerator;
import org.eu.adapt.demonstrator.web.ADAPTLookAndFeel;
import org.eu.adapt.demonstrator.web.Navigation;
import org.eu.adapt.services.demo_1_0.PurchaseOrderBasicService_wsdl.PurchaseOrderBasicServiceLocator;
import org.eu.adapt.services.demo_1_0.PurchaseOrderBasicService_wsdl.PurchaseOrderBasicServicePortType;
import org.eu.adapt.services.demo_1_0.PurchaseOrderBasicService_wsdl.types.PurchaseOrderStatus;
import org.eu.adapt.services.demo_1_0.RetailerService_wsdl.RetailerServiceLocator;
import org.ws_i.www.SampleApplications.SupplyChainManagement._2002_08.Configuration_xsd.ConfigurationType;
import org.ws_i.www.SampleApplications.SupplyChainManagement._2002_10.Manufacturer.CallBack.CallbackHeaderType;
import org.ws_i.www.SampleApplications.SupplyChainManagement._2002_10.ManufacturerSN_xsd.ShipmentNoticeType;
import org.ws_i.www.SampleApplications.SupplyChainManagement._2002_10.Manufacturer_wsdl.WarehouseCallbackPortType;

public class CheckStockServlet extends HttpServlet
{
    public void init()
        throws ServletException
    {
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException 
    {
        response.setContentType("text/html");
        response.setHeader("Cache-Control", "no-cache");

        PrintWriter writer = response.getWriter();

        ADAPTLookAndFeel.generatePageHeader("..", writer);
        Navigation.generateNavigationHeader("..", writer);

        updatePurchaseOrders(writer);
        
        Navigation.generateNavigationFooter(writer);
        ADAPTLookAndFeel.generatePageFooter(writer);
    }

    private void updatePurchaseOrders(PrintWriter writer)
    {
        try
        {
            String                            purchaseOrderBasicServiceURLString = getInitParameter("PurchaseOrderBasicServiceURL");
            PurchaseOrderBasicServiceLocator  purchaseOrderBasicServiceLocator   = new PurchaseOrderBasicServiceLocator();
            PurchaseOrderBasicServicePortType purchaseOrderBasicServicePortType  = purchaseOrderBasicServiceLocator.getPurchaseOrder(new URL(purchaseOrderBasicServiceURLString));

            String[] purchaseOrderNumbers = purchaseOrderBasicServicePortType.getListPurchaseOrders();

            for (int purchaseOrderIndex = 0; (purchaseOrderNumbers != null) && (purchaseOrderIndex < purchaseOrderNumbers.length); purchaseOrderIndex++)
                updatePurchaseOrder(purchaseOrderNumbers[purchaseOrderIndex], purchaseOrderBasicServicePortType, writer);
        }
        catch (Exception exception)
        {
            System.err.println("Exception in \"updatePurchaseOrder\": " + exception);
        }
        catch (Error error)
        {
            System.err.println("Error in \"updatePurchaseOrder\": " + error);
        }
    }

    private void updatePurchaseOrder(String purchaseOrderNumber, PurchaseOrderBasicServicePortType purchaseOrderBasicServicePortType, PrintWriter writer)
    {
        writer.println("<P><B>Purchase Order: \"" + purchaseOrderNumber + "\"</B> ");
        try
        {
            PurchaseOrderStatus purchaseOrderStatus = purchaseOrderBasicServicePortType.setUpdatePurchaseOrder(purchaseOrderNumber);

            if (PurchaseOrderStatus.ALREADY_COMPLETE.equals(purchaseOrderStatus))
                writer.println("Already complete</P>");
            else if (PurchaseOrderStatus.NOW_COMPLETE.equals(purchaseOrderStatus))
            {
                sendShipmentNotice(purchaseOrderNumber, "");
                writer.println("<B>Now complete</B></P>");
            }
            else if (PurchaseOrderStatus.NOT_COMPLETE.equals(purchaseOrderStatus))
                writer.println("Not complete</P>");
        }
        catch (Exception exception)
        {
            writer.println("<B>Failed</B></P>");
            System.err.println("Exception in \"produceStock\": " + exception);
        }
        catch (Error error)
        {
            writer.println("<B>Failed</B></P>");
            System.err.println("Error in \"produceStock\": " + error);
        }
    }

    private boolean sendShipmentNotice(String purchaseOrderNumber, String customerReference)
    {
        boolean done = false;

        try
        {
            String                    warehouseCallbackServiceURLString = getInitParameter("WarehouseCallbackServiceURL");
            RetailerServiceLocator    retailerServiceLocator            = new RetailerServiceLocator();
            WarehouseCallbackPortType warehouseCallbackPortType         = retailerServiceLocator.getWarehouseCallback(new URL(warehouseCallbackServiceURLString));

            ShipmentNoticeType shipmentNotice = new ShipmentNoticeType();
            ConfigurationType  configuration  = new ConfigurationType();
            CallbackHeaderType callbackHeader = new CallbackHeaderType(); 

            shipmentNotice.setOrderNum(purchaseOrderNumber);
// Causes Axis bug.
//            shipmentNotice.setCustomerRef(new CustomerReferenceType(customerReference)); 
            shipmentNotice.setShipNum(UniqueIdentifierGenerator.createUniqueIdentifier());

            done = warehouseCallbackPortType.submitSN(shipmentNotice, configuration, callbackHeader);
        }
        catch (Exception exception)
        {
            System.err.println("Exception in \"sendShipmentNotice\": " + exception);
            exception.printStackTrace(System.err);
            done = false;
        }
        catch (Error error)
        {
            System.err.println("Error in \"sendShipmentNotice\": " + error);
            done = false;
        }
        
        return done;
    }
}